   /**MEMBER**=SLCLIBS:MD386VAXLIB
    **CMS**=RMX_MD86VAX
   */

#if defined(VAXC) || defined(__DECC)
/* Following pragma packs data structure members on byte  */
/* boundaries while aligning the base of the structure on */
/* a longword (as required, e.g., for SLCNET buffers).    */
#pragma nomember_alignment LONGWORD
/* Following pragma kills some spurious error messages    */
/* caused by the preceding pragma.                        */
#pragma message disable (NONMULTALIGN)
#endif

/*************************************************************************/
/*                                                                       */
/*                     AfhFMutilities.c                                  */
/*                                                                       */
/*    This module contains utilities for FilterMeister.                  */
/*                                                                       */
/*    Copyright 1998-2014 AFH Systems                                    */
/*                                                                       */
/*  FilterMeister is free software. You can redistribute and modify it   */
/*  under the terms of the MPL2 License.                                 */
/*  See: https://www.mozilla.org/en-US/MPL/2.0/                          */
/*  Any changes to the FilterMeister files have to be published or sent  */
/*  to harry@thepluginsite.com. The filter plugin code itself does not   */
/*  need to be disclosed.                                                */
/*                                                                       */
/*************************************************************************/
/*                                                                       */
/*              R E V I S I O N   H I S T O R Y                          */
/*                                                                       */
/*  16Jan98  Alex Hunter  1. Derived from mde.c and mdo.c.               */
/*	21Jan14  Alex Hunter  1. Rename the PRINTF* macros to SPRINTF* to	 */
/*							 avoid  global naming conflict.				 */
/*	08Jun14	 Alex Hunter  1. Add !U*, !S*, !X* format codes, which are   */
/*							 the same as !UL, !SL, !XL except no chars   */
/*							 are printed if the corresponding arg is 0.  */
/*                                                                       */
/*************************************************************************/

#ifdef MDP_TEST
#include "AfhFM.h"                      /* msgBox.                                 */
#include "AfhFMsymtab.h"                /* FM symbol table definitions.            */
#include <windows.h>
#endif

#include "plmcvax.h"
#include <setjmp.h>                     /* longjmp, jmp_buf.                       */
#include "md386def.hc"                  /* MD386__ message codes.                  */
#include "md386lit.h"                   /* MD386 common definitions.               */
#include "mdg.h"                        /* Mdg_ctrl_c_abort.                       */
#include "mdd.h"
#include "mdy.h"                        /* QUALIFIERSTRUCTURE.                     */
#include "md386proto.h"                 /* MD386 intermodule function prototypes.  */

#include <stdio.h>                      /* sprintf.                                */
#include <ctype.h>                      /* toupper.                                */
#include <string.h>                     /* memcpy, memmove, memset, strcat,        */
                                        /*  strcpy, strlen, strncat.               */

        /********************************************/
        /*                                          */
        /*                                          */
        /*          Global Jump Buffers             */
        /*                                          */
        /*                                          */
        /********************************************/

 jmp_buf Mde_error_exit_jb;

/******************* Static data and variables ************************/

 static UINT1                  facilityname[] = "FM";
 static const STRINGDESCRIPTOR facilitynamed = {sizeof(facilityname)-1,
                                                facilityname};

    /* Formatted print routines for FM */

#ifdef FML

#ifdef MDP_TEST
 static char s[256];
 static int n;
#endif

void FMprint(const char *buff);

//BUFFER OVERFLOWS POSSIBLE BELOW!!!
#define SPRINTF(f) \
    do {\
        FMprint(f);\
        n += sprintf(&s[n], f);\
    } while(0)

#define SPRINTF2(f,a) \
    do {\
        char buff[128];\
        sprintf(buff, f, a);\
        FMprint(buff);\
        n += sprintf(&s[n], f, a);\
    } while(0)

#define SPRINTF3(f,a,b) \
    do {\
        char buff[128];\
        sprintf(buff, f, a, b);\
        FMprint(buff);\
        n += sprintf(&s[n], f, a, b);\
    } while(0)

#endif


#ifdef WIN32

        /**************************************/
        /*                                    */
        /*                                    */
        /* Temporary lib$sys_getmsg for WIN32.*/
        /*                                    */
        /*                                    */
        /**************************************/

 /**procedure**/
 void lib$sys_getmsg(const UINT4 *cvp, UINT2 *lenp, const STRINGDESCRIPTOR *msgdp,
                     const UINT4 *msgmaskp)
 {
     UINT4 msgmask = 15;  /* default mask */
     char buffer[512];  /* better be an upper bound! */
     UINT2 len;
     int i;
     static const struct
     {
         UINT4 code;
         char *id;
         char *text;
     } msgdef[] = {

#include "md386msg.h"

          0,        "UNKNOWN",  "unknown error message (X'!-!-!XL')"
     };

     /* Lookup the message code */
     for (i = 0; i < sizeof(msgdef)/sizeof(msgdef[0]) - 1 ; ++i)
         if (msgdef[i].code == *cvp) break;

     /* Set the message mask if specified */
     if (msgmaskp && *msgmaskp)
         msgmask = *msgmaskp;

     /* start with a null string */
     buffer[0] = 0;

     if (msgmask & 8)
     {   /* Include the facility name */
#ifndef MDP_TEST
         strcat(buffer, "%");
#endif
         strncat(buffer, facilitynamed.ptr, facilitynamed.len);
     }

     if (msgmask & 4)
     {   /* Include the severity */
         if (msgmask & 8)
             strcat(buffer, "-");
         switch (*cvp & 7)
         {
         case 0:
             strcat(buffer, "W");
             break;
         case 1:
             strcat(buffer, "S");
             break;
         case 2:
             strcat(buffer, "E");
             break;
         case 3:
             strcat(buffer, "I");
             break;
         case 4:
             strcat(buffer, "F");
             break;
         default:
             strcat(buffer, "?");
         }
     }

     if (msgmask & 2)
     {   /* Include the msg id */
         if (msgmask & 12)
             strcat(buffer, "-");
         strcat(buffer, msgdef[i].id);
     }

     if (msgmask & 1)
     {   /* Include the text */
         int n;

         if (msgmask & 14) {
#ifdef MDP_TEST
             strcat(buffer, ":!/");  //CRLF
#else
             strcat(buffer, ", ");
#endif
         }
         n = (int)strlen(buffer);    //n = index of terminating NUL
         strcat(buffer, msgdef[i].text);
#ifndef MDP_TEST
         if ((msgmask & 14) == 0)
#endif
         {   /* For text-only, capitalize the first letter */
             buffer[n] = toupper(buffer[n]);
         }
     }

     memset(msgdp->ptr, ' ', (UINT2)msgdp->len);
     len = (UINT2)strlen(buffer);
     if (len > (UINT2)msgdp->len)
         len = (UINT2)msgdp->len;
     memcpy(msgdp->ptr, buffer, len);
     *lenp = len;
 } /*lib$sys_getmsg*/


        /**************************************/
        /*                                    */
        /*                                    */
        /*  Temporary sys$putmsg for WIN32.   */
        /*                                    */
        /*                                    */
        /**************************************/

 /**procedure**/
 void sys$putmsg(UINT_PTR arglist[6], void logroutine(),
                 const STRINGDESCRIPTOR *facilitynamedp, int lastarg)
 {
     UINT4 msgmask;
     UINT2 len;
     char msg[257]; /* 256 chars + room for 1 NUL */
     const STRINGDESCRIPTOR msgd = { sizeof(msg) - 1, msg };
     UINT_PTR *argp;
     char *cp;
     UINT4 last_value = 0;

     msgmask = (UINT4)(arglist[2] >> 16);

     /* Get the FAO message text. */
     lib$sys_getmsg((UINT4 *)&arglist[1], &len, &msgd, &msgmask);

     /* NUL-terminate the FAO text. */
     msg[len] = 0;

     argp = &arglist[3];  /* point to P1 as first FAO arg */
     cp = msg;            /* point to first char of FAO text */

#ifdef MDP_TEST
     s[0] = 0;
     n = 0;
#endif

     while (*cp != 0)
     {
         if (*cp == '!')
         {   /* Possible FAO directive */
             ++cp;
             if (*cp == 'A')
             {   /* ASCII string directive */
                 ++cp;
                 if (*cp == 'C')
                 {   /* !AC -- bytecount-prefixed ASCII string */
                     //Mdputc((char *) *argp++);
                     SPRINTF3("%.*s", *(UINT1 *)argp[0], (char *)argp[0] + 1);
                     ++argp;
                 }
                 else if (*cp == 'D')
                 {   /* !AD -- (len, ptr) string descriptor */
                     //Mdputs((STRINGDESCRIPTOR *) argp);
                     SPRINTF3("%.*s", (UINT2)argp[0], (char *)argp[1]);
                     ++argp;  /* move past len */
                     ++argp;  /* move past ptr */
                 }
                 else if (*cp == 'S')
                 {   /* !AS -- ptr to string descriptor */
                     //Mdputs((STRINGDESCRIPTOR *) *argp++);
                  ///SPRINTF3("%.*s", *(UINT2 *)argp[0], (char *)((UINT4 *)argp[0])[1] );//x86 old
					 SPRINTF3("%.*s", (UINT2)(((STRINGDESCRIPTOR *)argp[0])->len),
					                          ((STRINGDESCRIPTOR *)argp[0])->ptr );//x64
                     ++argp;
                 }
                 else if (*cp == 'Z')
                 {   /* !AZ -- zero-terminated string */
                     //Mdputz((char *) *argp++);
                     //SPRINTF((char *)*argp++); //BUG!  SPRINTF evaluates its arg twice!!!
                     SPRINTF((char *)*argp);
                     ++argp;
                 }
                 else
                 {   /* Unknown/unimplemented -- output directive verbatim */
                     SPRINTF("!A");
                     --cp;  /* backup char ptr */
                 }
             }
             else if (*cp == 'U')
             {   /* UNSIGNED decimal directive */
                 ++cp;
                 if (*cp == 'B')
                 {   /* !UB -- unsigned decimal byte */
                     last_value = (UINT4)(*argp++) & 0xff;
                     //Mdputu(&last_value);
                     SPRINTF2("%u", last_value);
                 }
                 else if (*cp == 'W')
                 {   /* !UW -- unsigned decimal word */
                     last_value = (UINT4)(*argp++) & 0xffff;
                     //Mdputu(&last_value);
                     SPRINTF2("%u", last_value);
                 }
                 else if (*cp == 'L')
                 {   /* !UL -- unsigned decimal longword */
                     last_value = (UINT4)(*argp++);
                     //Mdputu(&last_value);
                     SPRINTF2("%u", last_value);
                 }
                 else if (*cp == '*')
                 {   /* !U* -- unsigned nonzero decimal longword */
                     last_value = (UINT4)(*argp++);
					 if (last_value != 0) {
						 //Mdputu(&last_value);
						 SPRINTF2("%u", last_value);
					 }
                 }
                 else
                 {   /* Unknown/unimplemented -- output directive verbatim */
                     SPRINTF("!U");
                     --cp;  /* backup char ptr */
                 }
             }
             else if (*cp == 'S')
             {   /* SIGNED decimal directive */
                 ++cp;
                 if (*cp == 'B')
                 {   /* !SB -- signed decimal byte */
                     last_value = (INT1)*argp++;
                     //Mdputd(&last_value);
                     SPRINTF2("%d", last_value);
                 }
                 else if (*cp == 'W')
                 {   /* !SW -- signed decimal word */
                     last_value = (INT2)*argp++;
                     //Mdputd(&last_value);
                     SPRINTF2("%d", last_value);
                 }
                 else if (*cp == 'L')
                 {   /* !SL -- signed decimal longword */
                     last_value = (INT4)*argp++;
                     //Mdputd(&last_value);
                     SPRINTF2("%d", last_value);
                 }
                 else if (*cp == '*')
                 {   /* !S* -- signed nonzero decimal longword */
                     last_value = (INT4)*argp++;
					 if (last_value != 0) {
						 //Mdputd(&last_value);
						 SPRINTF2("%d", last_value);
					 }
                 }
                 else
                 {   /* Unknown/unimplemented -- output directive verbatim */
                     SPRINTF("!S");
                     --cp;  /* backup char ptr */
                 }
             }
             else if (*cp == 'X')
             {   /* UNSIGNED HEX directive */
                 ++cp;
                 if (*cp == 'B')
                 {   /* !XB -- unsigned hex byte */
                     last_value = (UINT4)(*argp++) & 0xff;
                     //Mdputhb((UINT1 *) &last_value);
                     SPRINTF2("%.2x", last_value);
                 }
                 else if (*cp == 'W')
                 {   /* !XW -- unsigned hex word */
                     last_value = (UINT4)(*argp++) & 0xffff;
                     //Mdputhw((UINT2 *) &last_value);
                     SPRINTF2("%.4x", last_value);
                 }
                 else if (*cp == 'L')
                 {   /* !XL -- unsigned hex longword */
                     last_value = (UINT4)(*argp++);
                     //Mdputhdw(&last_value);
                     SPRINTF2("%.8x", last_value);
                 }
                 else if (*cp == '*')
                 {   /* !X* -- unsigned nonzero hex longword */
                     last_value = (UINT4)(*argp++);
					 if (last_value != 0) {
						 //Mdputhdw(&last_value);
						 SPRINTF2("%.8x", last_value);
					 }
                 }
                 else
                 {   /* Unknown/unimplemented -- output directive verbatim */
                     SPRINTF("!X");
                     --cp;  /* backup char ptr */
                 }
             }
             else if (*cp == '%')
             {   /* special formatting */
                 ++cp;
                 if (*cp == 'S')
                 {   /* !%S -- output 's' if last value != 1 */
                     if (last_value != 1)
                         SPRINTF("s");
                 }
                 else
                 {   /* Unknown/unimplemented -- output directive verbatim */
                     SPRINTF("!%");
                     --cp;  /* backup char ptr */
                 }
             }
             else if (*cp == '/')
             {   /* CR/LF */
                 //Mdputcr();
                 //Mdputnl();
                 SPRINTF("\r\n");
             }
             else if (*cp == '_')
             {   /* TAB */
                 SPRINTF("\t");
             }
             else if (*cp == '^')
             {   /* FORM FEED */
                 SPRINTF("\f");
             }
             else if (*cp == '!')
             {   /* literal '!' */
                 SPRINTF("!");
             }
             else if (*cp == '-')
             {   /* reuse last argument */
                 --argp;
             }
             else if (*cp == '+')
             {   /* skip next argument */
                 ++argp;
             }
             else
             {   /* Unknown/unimplemented -- output directive verbatim */
                 SPRINTF("!");
                 --cp;  /* backup char ptr */
             }
         }
         else
         {   /* Output plain char */
             //Mdput(cp);
             // This should be optimized to put out a large sequence at once...
             char *cpstart = cp;
             while (*cp != 0 && *cp != '!')
                 ++cp;
             SPRINTF3("%.*s", cp - cpstart, cpstart);
             --cp;
         }
         ++cp;
     } /*while*/

     //Mdputcr();
#ifdef MDP_TEST
     FMprint("\r\n");
#else
     SPRINTF("\r\n");
#endif
 } /*sys$putmsg*/

#endif


        /**************************************/
        /*                                    */
        /*                                    */
        /*  Put Formatted Message Routine     */
        /*                                    */
        /*                                    */
        /**************************************/

 /**procedure**/
 void Mdo_put_msg(UINT4 msgid, UINT4 msgflags, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3)
 {
     UINT_PTR arglist[6];

     arglist[0] = 5;
     arglist[1] = msgid;
     arglist[2] = (msgflags << 16) | 3;  /* 3 = # of FAO args (p1 - p3) */
     arglist[3] = p1;
     arglist[4] = p2;
     arglist[5] = p3;
 
     if ((arglist[1] & 0xFFFF0000) == 0)
         arglist[1] += 0x20000;         /* make non-SS and non-RMS */

     sys$putmsg(arglist, NULL, &facilitynamed, 0);
 } /*Mdo_put_msg*/



        /**************************************/
        /*                                    */
        /*                                    */
        /*  Error Reporting Routines          */
        /*                                    */
        /*                                    */
        /**************************************/

 /**procedure**/
 void Mde_fatal(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3)
 {
     Mdo_put_msg(msgid, 0, p1, p2, p3);
#ifdef MDP_TEST
     gParams->fValidCompile = FALSE;
     msgBox(MB_ICONERROR, "FilterMeister Fatal Error", s);
#endif
     longjmp(Mde_error_exit_jb, 1);
 } /*Mde_fatal*/


 /**procedure**/
 void Mde_error(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3)
 {
     int reply;

     Mdo_put_msg(msgid, 0, p1, p2, p3);
#ifdef MDP_TEST
     gParams->fValidCompile = FALSE;
     do {
         reply = msgBox(MB_ICONERROR|MB_ABORTRETRYIGNORE, "FilterMeister Error", s);
     } while (reply == IDRETRY); // hehe...
     if (reply == IDABORT) {
         longjmp(Mde_error_exit_jb, 1);
     }
#endif
 } /*Mde_error*/


 /**procedure**/
 void Mde_warn(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3)
 {
     int reply;

     Mdo_put_msg(msgid, 0, p1, p2, p3);
#ifdef MDP_TEST
     reply = msgBox(MB_ICONWARNING|MB_OKCANCEL, "FilterMeister Warning", s);
     if (reply == IDCANCEL) {
         Mde_abort_command(MD386__USERCAN, 0, 0, 0);
     }
#endif
 } /*Mde_warn*/


 /**procedure**/
 void Mde_log(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3)
 {
     Mdo_put_msg(msgid, 1, p1, p2, p3);
#ifdef MDP_TEST
     msgBox(MB_ICONINFORMATION, "FilterMeister", s);
#endif
 } /*Mde_log*/


 /**procedure**/
 void Mde_abort_command(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3)
 {
     Mdo_put_msg(msgid, 0, p1, p2, p3);
#ifdef MDP_TEST
     gParams->fValidCompile = FALSE;
     msgBox(MB_ICONERROR, "FilterMeister Error", s);
#endif
     longjmp(Mde_error_exit_jb, 1);
 } /*Mde_abort_command*/
