/*
	File: AfhPiPL.c
	C source file for the AFH FilterMeister PiPL stuff.

    Copyright 1999 AFH Systems.

	FilterMeister is free software. You can redistribute and modify it
    under the terms of the MPL2 License.
    See: https://www.mozilla.org/en-US/MPL/2.0/
    Any changes to the FilterMeister files have to be published or sent
    to harry@thepluginsite.com. The filter plugin code itself does not
    need to be disclosed.

*/

#if !defined(BETA) && !defined(STANDALONE)

#include <windows.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <stddef.h>     /* offsetof */

#include "AfhFM.h"
#include "PIFilter.h"

#define CNVTPIPL_MAGIC1 0x0001
#define CNVTPIPL_MAGIC_CSTRING 0x0003

#define INIT_PIPL_SIZE 2048
#define PIPL_SIZE_INCREMENT 2048

#define ADOBE_VENDOR_ID '8BIM'

// Turn off alignment for following structure definition...
#if WIN32
#pragma pack(push,1)
#endif

typedef struct {
    int16 cnvtpipl_magic;   // "Magic" number inserted by CNVTPIPL.
    PIPropertyList;         // Property list header.
} PIPLHEADER;

#if WIN32
#pragma pack(pop)
#endif

static PIProperty *pPipl;
static PIPLHEADER *pPiplHeader;
static int cbPiplSize;
static int cbBytesLeft;

    ///// CreatePipl /////

BOOL CreatePipl(void)
{
    // Initialize data structures to create a new PiPL resource in memory.
    // Returns TRUE if successful, else FALSE;
    pPiplHeader = malloc(INIT_PIPL_SIZE);
    if (pPiplHeader == NULL) return FALSE;
    // Fill in the PiPL header.
    pPiplHeader->cnvtpipl_magic = CNVTPIPL_MAGIC1;
    pPiplHeader->version = kCurrentPiPLVersion;
    pPiplHeader->count = 0;     // Number of properties to follow...
    pPipl = &pPiplHeader->properties[0];
    cbPiplSize = offsetof(PIPLHEADER, properties);
    cbBytesLeft = INIT_PIPL_SIZE - cbPiplSize;
    return TRUE;
} /*CreatePipl*/


    ///// AddPiplProp /////

BOOL AddPiplProp(PIType propertyKey, int32  propertyID, int32  propertyLength,
                 const void *propertyData)
{
    //Pad propertyData to a multiple of 4 bytes...
    int padCount = -propertyLength & 3;
    int nBytes = offsetof(PIProperty, propertyData) + propertyLength + padCount;

    if (nBytes > cbBytesLeft) {
        // Need more room!
        int cbSizeNeeded = cbPiplSize + nBytes;
        // Round up to a multiple of PIPL_SIZE_INCREMENT...
        int cbSizeWanted = ((cbSizeNeeded + PIPL_SIZE_INCREMENT - 1)/PIPL_SIZE_INCREMENT) *
                           PIPL_SIZE_INCREMENT;
        PIPLHEADER *pNewPiplHeader = realloc(pPiplHeader, cbSizeWanted);
        if (pNewPiplHeader == NULL) {
            free(pPiplHeader);
            return FALSE;
        }
        pPiplHeader = pNewPiplHeader;
        pPipl = (PIProperty *)((char *)pPiplHeader + cbPiplSize);
        cbBytesLeft = cbSizeWanted - cbPiplSize;
    }
    pPipl->vendorID = ADOBE_VENDOR_ID;  //Adobe is the vendor
    pPipl->propertyKey = propertyKey;
    pPipl->propertyID = propertyID;     //normally 0
#if 1
    //workaround for PiCo bug, which assumes propertyLength field is
    //always a multiple of 4...
    pPipl->propertyLength = propertyLength + padCount;
#else
    pPipl->propertyLength = propertyLength;
#endif
    memcpy(pPipl->propertyData, propertyData, propertyLength);
    pPiplHeader->count++;
    memset(&pPipl->propertyData[propertyLength], 0, padCount);
    pPipl = (PIProperty *)((char *)pPipl + nBytes);
    cbPiplSize += nBytes;
    cbBytesLeft -= nBytes;
    return TRUE;
} /*AddPiplProp*/


    ///// FinishPipl /////

char *FinishPipl(int *pcbPiplSize)
{
    if (pcbPiplSize != NULL) {
        *pcbPiplSize = cbPiplSize;
    }
    return (char *)pPiplHeader; //Caller must free this pointer after use!
} /*FinishPipl*/


#endif //#if !defined(BETA) && !defined(STANDALONE)
