/*********************************   MDP.H   ********************************/
/*                                                                          */
/*  Globally visible definitions for the FM Parser.                         */
/*                                                                          */
/*  Copyright 1997-2014 AFH Systems                                         */
/*                                                                          */
/*  FilterMeister is free software. You can redistribute and modify it      */
/*  under the terms of the MPL2 License.                                    */
/*  See: https://www.mozilla.org/en-US/MPL/2.0/                             */
/*  Any changes to the FilterMeister files have to be published or sent     */
/*  to harry@thepluginsite.com. The filter plugin code itself does not      */
/*  need to be disclosed.                                                   */
/*                                                                          */
/*    26Nov97  Alex Hunter  1. Initial version.                             */
/*    12Jan98  Alex Hunter  1. Move SDL definitions to mdp.h.               */
/*    05Apr98  Alex Hunter  1. Add parser flags.                            */
/*    01Sep12  Alex Hunter	1. Add minimal X64 support.						*/
/*	  16JUL13  Alex Hunter  1. Modify for WIN64.							*/
/*	  03OCT13  Alex Hunter  1. Add DCL_HALF, DCL_INT_PTR, DCL_UINT_PTR.		*/
/*							2. Add OP_HALF, OP_INT_PTR, OP_UINT_PTR.		*/
/*	  21OCT13  Alex Hunter  1. Change 'treal' to 'tempreal'.				*/
/*    12NOV13  Alex Hunter  1. Add LIT_UINT32, LIT_UINT64, LIT_UINT_PTR.    */
/*	  03MAR14  Alex Hunter  1. Add SD_PROG for program type and revision.	*/
/*	  06JUN14  Alex Hunter  1. Add DCL_BCD, OP_BCD, and OP_TEMPREAL.		*/
/*							2. Change DCL_TREAL to DCL_TEMPREAL;			*/
/*							3. Add OP_VOID.									*/
/*                                                                          */
/****************************************************************************/


/******************* Language Definitions ************************/

enum LANG { LANG_UNKNOWN,
            LANG_ASM,
            LANG_C,
            LANG_CPP,
            LANG_FORTRAN,
            LANG_PASCAL,
            LANG_PLM        };

/******************* Program Types **********************/

enum PROG { PROG_UNKNOWN,
            PROG_AFS,
            PROG_FFW,
            PROG_FFP,
            PROG_FML,
            PROG_NULL       };

/***************** Semantic Descriptor List Definitions ************/
/*
;  Semantic descriptor headers.
;
;  The first dword of a semantic descriptor item has the format:
;
;       +--------+--------+--------+--------+
;       |# bytes to follow|  type  |  kind  |
;       +--------+--------+--------+--------+
*/

/*** crackers ***/

#define SDKIND(sditem)		((sditem) & 0xff)
#define SDTYPE(sditem)		(((sditem) >> 8) & 0xff)
#define SDTYPEKIND(sditem)	((sditem) & 0xffff)
#define SDBYTES(sditem)		(((sditem) >> 16) & 0xffff)

/*** Kinds/Types ***/

// ID (kind=1)
/**deprecated!!! use SD_IDLIT mostly**/
#ifdef _WIN64
#define SD_ID           0x0C0001        /* identifier (12-byte string descriptor)*/
#else //Win32
#define SD_ID           0x080001        /* identifier (8-byte string descriptor)*/
#endif

// Literals (kind=2)
#define LIT_INTEGER     0x040102        /* literal integer*4 value */
#define LIT_LOGICAL     0x040202        /* literal logical*4 value */
#define LIT_REAL        0x080302        /* literal real*8 value */
#define LIT_CHAR        0x000402        /* literal character*n value (n to be filled in) */
#define LIT_REGION      0x040502        /* literal region (4 byte handle) */	//X64 INT_PTR??
#define LIT_INT64		0x080602		/* literal integer*8 value */	//OR REUSE type=1 with bytes=8, etc.?
#define LIT_UINT64		0x080702		/* literal unsigned integer*8 value */
#define LIT_UINT32		0x040802		/* literal unsigned integer*4 value */
#ifdef _WIN64							/* literal integer value size of a ptr */ //IS THIS NEEDED??
#define LIT_INT_PTR		LIT_INT64		/* 64-bit integer if WIN64 */
#define LIT_UINT_PTR	LIT_UINT64		/* 64-bit unsigned integer if WIN64 */
#else
#define LIT_INT_PTR		LIT_INTEGER		/* 32-bit integer if WIN32 */
#define LIT_UINT_PTR	LIT_UINT32		/* 32-bit unsigned integer if WIN32 */
#endif

// Operators (kind=3)
#define OP_ADD          0x000103        /* addition operator */
#define OP_SUB          0x000203        /* subtraction operator */
#define OP_NEG          0x000303        /* negation operator */
#define OP_LOAD         0x000403        /* load (dereference) operator */
#define OP_STORE        0x000503        /* store operator */
#define OP_QUAL         0x000603        /* qualification (path) operator */
#define OP_LOOKUP       0x000703        /* lookup operator */
#define OP_INDEX        0x000803        /* array index operator */
#define OP_MEMBER       0x000903        /* member name operator */
#define OP_CONCAT       0x000A03        /* string concatenation operator */
#define OP_BLDPTR       0x000B03        /* build pointer operator */
#define OP_DUPL         0x000C03        /* duplicate operator */
#define OP_PHYS         0x000D03        /* physical address operator */
#define OP_MARK         0x000E03        /* mark start of array indices */
#define OP_VAR2TYPE     0x000F03        /* set type from variable */
#define OP_LSH          0x001003        /* left shift operator */
#define OP_RSH          0x001103        /* right shift operator */
#define OP_MUL          0x001203        /* multiplication operator */
#define OP_DIV          0x001303        /* division operator */
#define OP_REM          0x001403        /* remainder operator */
#define OP_ASS_ADD      0x001503        /* += operator */
#define OP_ASS_SUB      0x001603        /* -= operator */
#define OP_ASS_MUL      0x001703        /* *= operator */
#define OP_ASS_DIV      0x001803        /* /= operator */
#define OP_ASS_REM      0x001903        /* %= operator */
#define OP_ASS_AND      0x001A03        /* &= operator */
#define OP_ASS_XOR      0x001B03        /* ^= operator */
#define OP_ASS_IOR      0x001C03        /* |= operator */
#define OP_ASS_LSH      0x001D03        /* <<= operator */
#define OP_ASS_RSH      0x001E03        /* >>= operator */
#define OP_LOG_AND      0x001F03        /* logical AND (&&) operator */
#define OP_LOG_OR       0x002003        /* logical OR (||) operator */
#define OP_LOG_NOT      0x002103        /* logical NOT (!) operator */
#define OP_IF           0x002203        /* conditional (?:) operator */
#define OP_AND          0x002303        /* bitwise AND operator */
#define OP_IOR          0x002403        /* bitwise inclusive-OR operator */
#define OP_XOR          0x002503        /* bitwise exclusive-OR operator */
#define OP_NOT          0x002603        /* bitwise NOT operator */
#define OP_PREINC       0x002703        /* pre-increment operator */
#define OP_PREDEC       0x002803        /* pre-decrement operator */
#define OP_POSTINC      0x002903        /* post-increment operator */
#define OP_POSTDEC      0x002A03        /* post-decrement operator */
#define OP_ARG          0x002B03        /* push argument on call stack */
#define OP_CALL         0x002C03        /* CALL operator */
#define OP_SIZEOF       0x002D03        /* C sizeof operator */
#define OP_ADDR         0x002E03        /* 'address-of' operator */
#define OP_NOP          0x002F03        /* no-operation operator */
#define OP_EQ           0x003003        /* relational operator (==) */
#define OP_NE           0x003103        /* relational operator (!=) */
#define OP_LT           0x003203        /* relational operator (<) */
#define OP_GT           0x003303        /* relational operator (>) */
#define OP_LE           0x003403        /* relational operator (<=) */
#define OP_GE           0x003503        /* relational operator (>=) */
#define OP_CAST         0x003603        /* C-language cast operator */
#define OP_THEN         0x003703        /* start of true part */
#define OP_ELSE         0x003803        /* start of false part */
#define OP_POP          0x003903        /* pop (discard) value from stack */
#define OP_XCHG         0x003A03        /* exchange top two stack elements */
#define OP_LOOP         0x003B03        /* start of loop */
#define OP_ENDLOOP      0x003C03        /* end of loop */
#define OP_BREAK        0x003D03        /* exit loop or switch */
#define OP_CONTINUE     0x003E03        /* continue to next loop iteration */
#define OP_NEXT         0x003F03        /* start of next loop iteration */
#define OP_GOTO         0x004003        /* go to label */
#define OP_LABEL        0x004103        /* define label here*/
#define OP_RETURN       0x004203        /* return from function */
#define OP_SWITCH       0x004303        /* start of switch */
#define OP_CASE         0x004403        /* start of case expression */
#define OP_CASETEST     0x004503        /* end of case expression */
#define OP_DEFAULT      0x004603        /* start of default case */
#define OP_ENDSW        0x004703        /* end of switch */
#define OP_BREAKF       0x004803        /* exit loop or switch if FALSE */
#define OP_FOR          0x004903        /* start of for-loop */
#define OP_ENDFOR       0x004A03        /* end of for-loop */
#define OP_REPEAT       0x004B03        /* go to top of loop */
#define OP_SKIP         0x004C03        /* skip to end-skip */
#define OP_ENDSKIP      0x004D03        /* end of skip */
#define OP_DECLARE      0x004E03        /* start of declaration */
#define OP_ENTER        0x004F03        /* enter id into symbol table */
// Following must be contiguous values!!
#define OP_SETCHAN_R    0x005003        /* set vars for channel R */
#define OP_SETCHAN_G    0x005103        /* set vars for channel G */
#define OP_SETCHAN_B    0x005203        /* set vars for channel B */
#define OP_SETCHAN_A    0x005303        /* set vars for channel A */
#define OP_SETCHAN_S    0x005403        /* set vars for channel S */
#define OP_SETVAL_R     0x005503        /* set value of channel R */
#define OP_SETVAL_G     0x005603        /* set value of channel G */
#define OP_SETVAL_B     0x005703        /* set value of channel B */
#define OP_SETVAL_A     0x005803        /* set value of channel A */
#define OP_SETVAL_S     0x005903        /* set value of channel S */

#ifndef NO_FFP_INLINES
#define OP_MIN          0x005A03        /* FFP: inline min() */
#define OP_MAX          0x005B03        /* FFP: inline max() */
#define OP_SQR          0x005C03        /* FFP: inline sqr() */
#define OP_ABS          0x005D03        /* FFP: inline abs() */
#define OP_DIF          0x005E03        /* FFP: inline dif() */
#define OP_CTL          0x005F03        /* FFP: inline ctl() */
#define OP_VAL          0x006003        /* FFP: inline val() */
#define OP_SIN          0x006103        /* FFP: inline sin() */
#define OP_COS          0x006203        /* FFP: inline cos() */
#define OP_TAN          0x006303        /* FFP: inline tan() */
#define OP_GET          0x006403        /* FFP: inline get() */
#define OP_PUT          0x006503        /* FFP: inline put() */
#define OP_CNVX         0x006603        /* FFP: inline cnvX() */
#define OP_CNVY         0x006703        /* FFP: inline cnvY() */
#define OP_GAMMA        0x006803        /* FFP: inline gamma() */
#define OP_FTOL         0x006903        /* FFP: inline _ftol() */
#define OP_FLOOR        0x006A03        /* FFP: inline floor() */
#define OP_CEIL         0x006B03        /* FFP: inline ceil() */
#define OP_ROUND        0x006C03        /* FFP: inline round() */
#define OP_CHOP         0x006D03        /* FFP: inline chop() */
#define OP_IFLOOR       0x006E03        /* FFP: inline ifloor() */
#define OP_ICEIL        0x006F03        /* FFP: inline iceil() */
#define OP_IROUND       0x007003        /* FFP: inline iround() */
#define OP_ICHOP        0x007103        /* FFP: inline ichop() */
#define OP_POW          0x007203        /* FFP: inline pow() */
#endif //!NO_FFP_INLINES

#define OP_ON_END              0x008003    /* FFP: end of OnEvent handler (sets/restores ebx) */
#define OP_ON_INIT_DIALOG      0x008103    /* FFP: OnInitDialog */
#define OP_ON_CTL              0x008203    /* FFP: OnCtl */
#define OP_ON_ZERO_DIVIDE      0x008303    /* FFP: OnZeroDivide */
#define OP_ON_DIVIDE_OVERFLOW  0x008403    /* FFP: OnDivideOverflow */
#define OP_FOR_EVERY_PIXEL     0x008503    /* FFP: ForEveryPixel */
#define OP_FOR_EVERY_PIXEL_END 0x008603    /* FFP: end of ForEveryPixel handler */
#define OP_FOR_EVERY_TILE      0x008703    /* FFP: ForEveryTile */
#define OP_FOR_EVERY_END       0x008803    /* FFP: end of ForEvery* handler */
#define OP_ON_HANDLER_END      0x008903    /* FFP: end of OnEvent handler (ebx already set) */
#define OP_ON_FILTER_START     0x008A03    /* FFP: OnFilterStart */
#define OP_ON_FILTER_END       0x008B03    /* FFP: OnFilterEnd */
#define OP_USER_FUNCTION       0x008C03    /* FFP: UserFunction */

#define OP_SAVE_CHANNEL_CODE   0x009003    /* save channel code in buffer */
#define OP_END_CHANNEL_CODE    0x009103    /* end channel code buffer */
#define OP_DUMP_CHANNEL_CODE   0x009203    /* dump channel code back to main buffer */

#define OP_FOR1                0x009303    /* FOR-loop part 1 */
#define OP_FOR2                0x009403    /* FOR-loop part 2 */
#define OP_FOR3                0x009503    /* FOR-loop part 3 */
#define OP_FOR4                0x009603    /* FOR-loop part 4 */

#define OP_DO1                 0x009703    /* DO-loop part 1 */
#define OP_DO2                 0x009803    /* DO-loop part 2 */
#define OP_DO3                 0x009903    /* DO-loop part 3 */

#define OP_COMPOSE_TYPE		   0x00A003    /* compose type specifiers */
#define OP_COMPOSE_DECL        0x00A103    /* compose declarators */
#define OP_END_DECL            0x00A203    /* end of declaration */
#define OP_BEGIN_SCOPE         0x00A303    /* begin symbol scope */
#define OP_END_SCOPE           0x00A403    /* end symbol scope */
#define OP_BEGIN_INIT          0x00A503    /* begin <initializer> */
#define OP_END_INIT            0x00A603    /* end of <initializer> */

// Following are temporary until full OP_CAST operator is implemented...
#define OP_SHORT		       0x00B003    /* (short) */
#define OP_INT                 0x00B103    /* (int) */
#define OP_LONG                0x00B203    /* (long) */
#define OP_BOOL                0x00B303    /* (bool) */
#define OP_FLOAT               0x00B403    /* (float) */
#define OP_DOUBLE              0x00B503    /* (double) */
#if X64
#define OP_INT64		       0x00B603	   /* (int64) */
#define OP_QWORD		       0x00B703	   /* (qword) */
#define OP_INT_PTR			   0x00B803	   /* (INT_PTR) */
#define OP_UINT_PTR			   0x00B903	   /* (UINT_PTR) */
#endif //X64
#define OP_HALF				   0x00BA03	   /* (half) */
#define OP_BYTE				   0x00BB03	   /* (byte) */
#define OP_SIGNED			   0x00BC03	   /* (signed) */
#define OP_UNSIGNED			   0x00BD03	   /* (unsigned) */
#define OP_CHAR				   0x00BE03	   /* (char) */
#define OP_WORD				   0x00BF03    /* (word) */
#define OP_DWORD			   0x00C003    /* (dword) */
#define OP_TEMPREAL			   0x00C103	   /* (tempreal) */
#define OP_BCD				   0x00C203    /* (bcd) */
#define OP_VOID				   0x00C303	   /* (void) */

//More operators...
#define OP_DEC_STACK           0x00D003    /* Decrement stack w/o generating code */
#define OP_ARGLIST             0x00D103    /* Start of argument list */

//Following is temporary for Embed: key...
#define OP_EMBED_BITMAP        0x00E003    /* Embed:Bitmap */
#define OP_EMBED_ICON          0X00E103    /* Embed:Icon */
#define OP_EMBED_METAFILE      0x00E203    /* Embed:Metafile */
#define OP_EMBED_WAVE          0x00E303    /* Embed:Wave */
#define OP_EMBED_OTHER         0x00E403    /* Embed:Other */
#define OP_EMBED_CURSOR        0x00E503    /* Embed:Other */

//Following is temporary until full About: key is implemented...
#define OP_ABOUT               0x00F003    /* About text */

// Types (kind=4)
#define TYP_BYTE        0x040004        /* unsigned integer type */
#define TYP_INTEGER     0x040104        /* signed integer type */
#define TYP_LOGICAL     0x040204        /* logical type */
#define TYP_REAL        0x040304        /* real type */
#define TYP_CHAR        0x040404        /* character type */
#define TYP_INSTR       0x040504        /* machine instruction type */
#define TYP_PTR         0x040604        /* pointer type */
#define TYP_SELECTOR    0x040704        /* selector type */
#define TYP_INT64		0x040804		/* signed 64-bit integer type */
#define TYP_ARRAY       0x041004        /* array type */
#define TYP_STRUCTURE   0x041104        /* structure type */
#define TYP_LIST        0x041204        /* list type */
#define TYP_ASCIC       0x041304        /* counted ASCII string */
#define TYP_ASCIZ       0x041404        /* NUL-terminated ASCII string */
#define TYP_NULL        0x04FF04        /* null type */

// Semantic Descriptors (kind=5 thru 0x11)  NO LONGER USED???
#define SD_REG          0x040005        /* machine register */
#define SD_LINE         0x040006        /* line number */
#define SD_LABEL        0x040007        /* fortran label */
#ifdef _WIN64
#define SD_SEGMENT      0x0C0008        /* segment name (12-byte string descriptor)*/
#define SD_GROUP        0x0C0009        /* group name (12-byte string descriptor)*/
#define SD_COMMON       0x0C000A        /* common block name (12-byte string descriptor)*/
#else //Win32
#define SD_SEGMENT      0x080008        /* segment name (8-byte string descriptor)*/
#define SD_GROUP        0x080009        /* group name (8-byte string descriptor)*/
#define SD_COMMON       0x08000A        /* common block name (8-byte string descriptor)*/
#endif
#define SD_RADIX        0x04000B        /* numeric radix */
#define SD_MODE         0x04000C        /* mode (for SET MODE...) */
#ifdef _WIN64
#define SD_FILE         0x0C000D        /* file name (12-byte string descriptor)*/
#else //Win32
#define SD_FILE         0x08000D        /* file name (8-byte string descriptor)*/
#endif
#define SD_PC           0x00050E        /* current program counter ($) */
#define SD_LAST         0x00000F        /* current examine/deposit location (.) */
#define SD_NEXT         0x000010        /* next examine/deposit location */
#define SD_TIME         0x000111        /* absolute or relative time in DCL format */

// Declarations (kind=0x12)
#define DCL_TYPEDEF     0x000112        /* typedef */
#define DCL_EXTERN      0x000212        /* extern */
#define DCL_STATIC      0x000312        /* static */
#define DCL_AUTO        0x000412        /* auto */
#define DCL_REGISTER    0x000512        /* register */
#define DCL_VOID        0x000612        /* void */
#define DCL_CHAR        0x000712        /* char */
#define DCL_SHORT       0x000812        /* short */
#define DCL_INT         0x000912        /* int */
#define DCL_LONG        0x000A12        /* long */
#define DCL_BOOL        0x000B12        /* bool */
#define DCL_BYTE        0x000C12        /* byte */
#define DCL_WORD        0x000D12        /* word */
#define DCL_DWORD       0x000E12        /* dword */
#define DCL_FLOAT       0x000F12        /* float */
#define DCL_DOUBLE      0x001012        /* double */
#define DCL_SIGNED      0x001112        /* signed */
#define DCL_UNSIGNED    0x001212        /* unsigned */
#define DCL_SATURATED   0x001312        /* saturated */
#define DCL_STRUCT      0x001412        /* struct */
#define DCL_UNION       0x001512        /* union */
#define DCL_ENUM        0x001612        /* enum */
#define DCL_CONST       0x001712        /* const */
#define DCL_VOLATILE    0x001812        /* volatile */
#define DCL_POINTER     0x001912        /* pointer (*) */
#define DCL_ARRAYDIM    0x001A12        /* array dimension */
#define DCL_PARAM       0x001B12        /* formal parameter */
#define DCL_MEMBER      0x001C12        /* struct, union, or enum member */
#define DCL_TEMPREAL    0x001D12        /* tempreal */
#define DCL_ARRAYDIMNUL 0x001E12        /* null array dimension [] */
#define DCL_FUNCTION    0x001F12        /* function */
#define DCL_ELLIPSIS    0x002012        /* ellipsis (...) */
#if X64
#define DCL_INT64		0x002112		/* int64 or _int64 */
#define DCL_QWORD		0x002212		/* qword */
#define DCL_INT_PTR		0x002312		/* INT_PTR */
#define DCL_UINT_PTR	0x002412		/* UINT_PTR */
#endif //X64
#define DCL_HALF		0x002512		/* half */
#define DCL_BCD			0x002612		/* bcd */

// Literal ID (kind=0x13) Supercedes SD_ID in most cases.
#define SD_IDLIT        0x000013        /* ID stored literally within semantic */
                                        /* descriptor item, a la LIT_CHAR. */

// Program type and revision (kind=0x14).
#define SD_PROG			0x0C0014		/* Prog type in param1,      */
										/* Prog rev major in param2, */
										/* Prog rev minor in param3. */
// Last used SD kind
#define LAST_VALID_SD   SD_PROG			/* Update whenever a new kind added!! */

/* parser flags */

#define MDP_USE_INLINES 0x00000001

/************************ end of MDP.H *********************/
